@@ -22,6 +22,7 @@
 #include <SDL/SDL.h>
 #include <glib.h>
 #include <pthread.h>
+//#include <gst/gst.h>
 
 #include "maemo.h"
 #include "usbjoy.h"
@@ -34,7 +35,7 @@
 
 int quickSlot;
 
-extern osso_events( gboolean block );
+extern gboolean osso_events( gboolean block );
 
 static int running = 0;
 static pthread_t sdl_thread;
@@ -50,15 +51,14 @@ static unsigned long keystates;
 static unsigned long mousestates;
 static unsigned long accelstates;
 
-static SDL_mutex* mutex;
-
 static unsigned short ts_key_layout[800*480];
 
 static void sdl_key( int key, int press ) {
-	unsigned short cmd = key_map[key];
+	unsigned long cmd = currentConfig.player[0].keymap[key];
 	if( cmd < 0x8000 ) {
-		if( currentConfig.enableKey )
+		if( currentConfig.player[0].inputFlags & _P_ENABLE_KEY ) {
 			if( press ) keystates |= cmd; else keystates &= ~cmd;
+		}
 	} else switch( cmd ) {
 		case CMD_QUIT:
 			running = 0;
@@ -73,13 +73,18 @@ static void sdl_key( int key, int press ) {
 			quickSlot = cmd & 0x03;
 			break;
 	}
+	cmd = currentConfig.player[1].keymap[key];
+	if( cmd < 0x8000 && currentConfig.player[1].inputFlags & _P_ENABLE_KEY ) {
+		cmd <<= 16;
+		if( press ) keystates |= cmd; else keystates &= ~cmd;
+	}
 }
 
 static void sdl_mouse( int x, int y, int mode ) {
-	if( !currentConfig.enableTS ) return;
 	if( ! (mode & 0x01) ) mousestates = 0;
 	if( !mode ) return;
-#define set( cmd ) mousestates |= cmd
+	unsigned long tmpmouse = 0;
+#define set( cmd ) tmpmouse |= cmd
 	if( x >= 0 && x <= 300 && y >= 0 && y <= 480 ) {
 		if( x >= 0 && x <= 100 ) set( MDB_LEFT );
 		if( x >= 200 && x <= 300 ) set( MDB_RIGHT );
@@ -90,21 +95,32 @@ static void sdl_mouse( int x, int y, int mode ) {
 		else if( y <= 320 ) set( MDB_B );
 		else set( MDB_C );
 	}
+	if( currentConfig.player[0].inputFlags & _P_ENABLE_TS ) mousestates |= tmpmouse;
+	if( currentConfig.player[1].inputFlags & _P_ENABLE_TS ) mousestates |= tmpmouse << 16;
 }
 
 static void read_accelerometer() {
-	if( !currentConfig.enableAccel ) return;
-#define ACCEL_TOLERANCE currentConfig.accelSensitivity
+	if( ! (currentConfig.player[0].inputFlags & _P_ENABLE_ACC || currentConfig.player[1].inputFlags & _P_ENABLE_ACC) ) return;
 	accelstates = 0;
 	int x, y, z;
 	FILE* f = fopen( "/sys/class/i2c-adapter/i2c-3/3-001d/coord", "r" );
 	if( !f ) return;
 	fscanf( f, "%d %d %d", &x, &y, &z );
 	fclose( f );
-	if( x > ACCEL_TOLERANCE ) accelstates |= MDB_LEFT;
-	else if( x < -ACCEL_TOLERANCE ) accelstates |= MDB_RIGHT;
-	if( y > ACCEL_TOLERANCE ) accelstates |= MDB_UP;
-	else if( y < -ACCEL_TOLERANCE ) accelstates |= MDB_DOWN;
+	if( currentConfig.player[0].inputFlags & _P_ENABLE_ACC ) {
+		int sens = currentConfig.player[0].accelSensitivity;
+		if( x > sens ) accelstates |= MDB_LEFT;
+		else if( x < -sens ) accelstates |= MDB_RIGHT;
+		if( y > sens ) accelstates |= MDB_UP;
+		else if( y < -sens ) accelstates |= MDB_DOWN;
+	}
+	if( currentConfig.player[1].inputFlags & _P_ENABLE_ACC ) {
+		int sens = currentConfig.player[1].accelSensitivity;
+		if( x > sens ) accelstates |= MDB_LEFT << 16;
+		else if( x < -sens ) accelstates |= MDB_RIGHT << 16;
+		if( y > sens ) accelstates |= MDB_UP << 16;
+		else if( y < -sens ) accelstates |= MDB_DOWN << 16;
+	}
 }
 
 static void* SDL_Loop( void* obj ) {
@@ -122,7 +138,8 @@ static void* SDL_Loop( void* obj ) {
 				sdl_mouse( event.button.x, event.button.y, 2 );
 				break;
 			case SDL_KEYDOWN:
-				sdl_key( event.key.keysym.scancode, 1 );
+				if( event.key.keysym.sym == SDLK_ESCAPE && event.key.keysym.mod & KMOD_CTRL ) maemo_quit();
+				else sdl_key( event.key.keysym.scancode, 1 );
 				break;
 			case SDL_KEYUP:
 				sdl_key( event.key.keysym.scancode, 0 );
@@ -140,7 +157,6 @@ static void* SDL_Loop( void* obj ) {
 }
 
 static void audio_loop( void* userdata, Uint8* stream, int len ) {
-	SDL_mutexP( mutex );
 	register int ptrr = soundptrr, ptrw = soundptrw;
 	int total = ptrw - ptrr;
 	if( total < 0 ) {
@@ -154,7 +170,6 @@ static void audio_loop( void* userdata, Uint8* stream, int len ) {
 	total = min( total, len );
 	memcpy( stream, sound_buffer + ptrr, total );
 	soundptrr = ( ptrr + total ) % SND_BUFFER_SIZE;
-	SDL_mutexV( mutex );
 }
 
 void maemo_quit() {
@@ -163,6 +178,23 @@ void maemo_quit() {
 }
 
 /* video stuff */
+
+static inline void disp_ts_buttons() {
+	register int i, j;
+	register unsigned short* buffer = screen->pixels;
+	for( i = 1; i <= 3; i++ )
+		for( j = 0; j < 480; j+=2 )
+			buffer[j*800+i*100] = 0xFFFF;
+	for( i = 1; i < 3; i++ )
+		for( j = 0; j < 300; j+=2 )
+			buffer[j+i*160*800] = 0xFFFF;
+	for( i = 0; i < 480; i+=2 )
+		buffer[i*800+700] = 0xFFFF;
+	for( i = 1; i < 3; i++ )
+		for( j = 700; j < 800; j+=2 )
+			buffer[i*160*800+j] = 0xFFFF;
+}
+
 void maemo_video_flip(void)
 {
 #define clerp16(a,b) ( ( ( ( a >> 12 ) + ( b >> 12 ) ) & 0x1F ) << 11 | ( ( ( a >> 6 ) + ( b >> 6 ) ) & 0x3F ) << 5 | ( ( ( a >> 1 ) + ( b >> 1 ) ) & 0x1F ) )
@@ -171,7 +203,6 @@ void maemo_video_flip(void)
 	unsigned short *frame = screen->pixels;
 	register unsigned short *scan1 = frame;
 	register unsigned short *scan2 = frame;
-	if( currentConfig.displayTS ) memcpy( frame, ts_key_layout, sizeof(ts_key_layout) );
 	switch( currentConfig.scaling ) {
 	case 1:
 		for( j = 0; j < 240; j++ ) {
@@ -234,6 +265,7 @@ void maemo_video_flip(void)
 		}
 		break;
 	}
+	if( currentConfig.displayTS ) disp_ts_buttons(); //memcpy( frame, ts_key_layout, sizeof(ts_key_layout) );
 	SDL_Flip(screen);
 }
 
@@ -274,7 +306,6 @@ void maemo_start_sound(int rate, int bits, int stereo)
 
 void maemo_sound_write(void *buff, int len)
 {
-	SDL_mutexP( mutex );
 	register int ptrr = soundptrr, ptrw = soundptrw;
 	register int total = ptrr - ptrw - 1;
 	if( total < 0 ) {
@@ -287,7 +318,6 @@ void maemo_sound_write(void *buff, int len)
 	total = min( len, total );
 	memcpy( sound_buffer + ptrw, buff, total );
 	soundptrw = ( ptrw + total ) % SND_BUFFER_SIZE;
-	SDL_mutexV( mutex );
 }
 
 void maemo_sound_sync(void)
@@ -324,7 +354,7 @@ void maemo_init(void)
 	printf("entering init()\n"); fflush(stdout);
 
 	SDL_Init( SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_EVENTTHREAD );
-	SDL_WM_SetCaption( "PicoDrive for Maemo 1.0", NULL );
+	SDL_WM_SetCaption( "PicoDrive for Maemo " G_STRINGIFY(GAME_VERSION), NULL );
 	SDL_ShowCursor( SDL_DISABLE );
 	SDL_WM_GrabInput( SDL_GRAB_OFF );
 
@@ -337,9 +367,7 @@ void maemo_init(void)
 
 	keystates = mousestates = accelstates = 0;
 	/* init usb joys -GnoStiC */
-	maemo_usbjoy_init();
-
-	mutex = SDL_CreateMutex();
+	//maemo_usbjoy_init();
 
 	running = 1;
 	pthread_create( &sdl_thread, NULL, SDL_Loop, NULL );
@@ -354,12 +382,11 @@ void maemo_deinit(void)
 	running = 0;
 	pthread_join( sdl_thread, NULL );
 
-	maemo_usbjoy_deinit();
+	//maemo_usbjoy_deinit();
 
 	SDL_PauseAudio(1);
 	SDL_CloseAudio();
 
-	SDL_DestroyMutex( mutex );
 	free( md_screen );
 
 	SDL_FreeSurface( screen );
