@@ -2,8 +2,6 @@
 #ifndef __MAEMO_H__
 #define __MAEMO_H__
 
-unsigned short key_map[0x100];
-
 void maemo_init(void);
 void maemo_deinit(void);
 void maemo_quit();
@@ -34,7 +32,7 @@ enum MDButton {
 	MDB_X		= 0x100,
 	MDB_Y		= 0x200,
 	MDB_Z		= 0x400,
-	MDB_M		= 0x800,
+	MDB_MODE	= 0x800,
 	CMD_QUIT 	= 0x8000,
 	CMD_QUICKLOAD = 0x8008,
 	CMD_QUICKSAVE = 0x800C
