@@ -25,6 +25,7 @@
 #define GPLAYER2_KEY kGConfPath "/player2" kGConfPlayerKeyboardPath "/"
 
 #define GPLAYER1_K kGConfPath "/player1"
+#define GPLAYER2_K kGConfPath "/player2"
 
 int use_gui = 0;
 char **g_argv;
@@ -42,9 +43,61 @@ int check_file( char* filename ) {
 	return 1;
 }
 
+void cli_config( char* file ) {
+	FILE* f = fopen( file, "r" );
+	if( !f ) {
+		printf( "Configuration file not found: %s\n", file );
+		if( !use_gui ) {
+			printf( "Cannot use default options, exiting\n" );
+			exit(1);
+		}
+		return;
+	}
+	printf( "Reading config file %s\n", file );
+	char opt[64], value[64];
+	unsigned short* key_map = currentConfig.player[0].keymap;
+	while( !feof(f) ) {
+		fscanf( f, "%s %s\n", opt, value );
+		if( opt[0] == '#' ) {
+			while( fgetc(f) != '\n' );
+			continue;
+		}
+		if( !strcmp( opt, "P1_K_A" ) ) key_map[atoi(value)] = MDB_A;
+		else if( !strcmp( opt, "P1_K_B" ) ) key_map[atoi(value)] = MDB_B;
+		else if( !strcmp( opt, "P1_K_C" ) ) key_map[atoi(value)] = MDB_C;
+		else if( !strcmp( opt, "P1_K_X" ) ) key_map[atoi(value)] = MDB_X;
+		else if( !strcmp( opt, "P1_K_Y" ) ) key_map[atoi(value)] = MDB_Y;
+		else if( !strcmp( opt, "P1_K_Z" ) ) key_map[atoi(value)] = MDB_Z;
+		else if( !strcmp( opt, "P1_K_UP" ) ) key_map[atoi(value)] = MDB_UP;
+		else if( !strcmp( opt, "P1_K_DOWN" ) ) key_map[atoi(value)] = MDB_DOWN;
+		else if( !strcmp( opt, "P1_K_LEFT" ) ) key_map[atoi(value)] = MDB_LEFT;
+		else if( !strcmp( opt, "P1_K_RIGHT" ) ) key_map[atoi(value)] = MDB_RIGHT;
+		else if( !strcmp( opt, "P1_K_UP_LEFT" ) ) key_map[atoi(value)] = MDB_UP | MDB_LEFT;
+		else if( !strcmp( opt, "P1_K_DOWN_RIGHT" ) ) key_map[atoi(value)] = MDB_DOWN | MDB_RIGHT;
+		else if( !strcmp( opt, "P1_K_DOWN_LEFT" ) ) key_map[atoi(value)] = MDB_LEFT | MDB_DOWN;
+		else if( !strcmp( opt, "P1_K_UP_RIGHT" ) ) key_map[atoi(value)] = MDB_RIGHT | MDB_RIGHT;
+		else if( !strcmp( opt, "P1_K_START" ) ) key_map[atoi(value)] = MDB_START;
+		else if( !strcmp( opt, "P1_K_MODE" ) ) key_map[atoi(value)] = MDB_MODE;
+		else if( !strcmp( opt, "CMD_MENU" ) ) key_map[atoi(value)] = CMD_QUIT;
+		else if( !strcmp( opt, "CMD_QUICKSAVE_1" ) ) key_map[atoi(value)] = CMD_QUICKSAVE;
+		else if( !strcmp( opt, "CMD_QUICKSAVE_2" ) ) key_map[atoi(value)] = CMD_QUICKSAVE + 1;
+		else if( !strcmp( opt, "CMD_QUICKSAVE_3" ) ) key_map[atoi(value)] = CMD_QUICKSAVE + 2;
+		else if( !strcmp( opt, "CMD_QUICKSAVE_4" ) ) key_map[atoi(value)] = CMD_QUICKSAVE + 3;
+		else if( !strcmp( opt, "CMD_QUICKLOAD_1" ) ) key_map[atoi(value)] = CMD_QUICKLOAD;
+		else if( !strcmp( opt, "CMD_QUICKLOAD_2" ) ) key_map[atoi(value)] = CMD_QUICKLOAD + 1;
+		else if( !strcmp( opt, "CMD_QUICKLOAD_3" ) ) key_map[atoi(value)] = CMD_QUICKLOAD + 2;
+		else if( !strcmp( opt, "CMD_QUICKLOAD_4" ) ) key_map[atoi(value)] = CMD_QUICKLOAD + 3;
+		else if( !strcmp( opt, "SCALING" ) ) currentConfig.scaling = atoi(value);
+		else if( !strcmp( opt, "FRAMESKIP" ) ) currentConfig.Frameskip = atoi(value);
+	}
+	currentConfig.screensaver = 0;
+	currentConfig.player[0].inputFlags = _P_ENABLE_KEY | _P_ENABLE_TS;
+}
+
 void parse_cmd_line(int argc, char *argv[])
 {
 	int x, unrecognized = 0;
+	char* conf_file = "picodrive.ini";
 
 	for(x = 1; x < argc; x++)
 	{
@@ -53,6 +106,10 @@ void parse_cmd_line(int argc, char *argv[])
 			if(strcasecmp(argv[x], "-gui") == 0) {
 				use_gui = 1;
 			}
+			if(strcasecmp(argv[x], "-o") == 0) {
+				if( argc > x+1 ) conf_file = argv[++x];
+				else unrecognized = 1;
+			}
 			else {
 				unrecognized = 1;
 				break;
@@ -61,6 +118,7 @@ void parse_cmd_line(int argc, char *argv[])
 			/* External Frontend: ROM Name */
 			strncpy(romFileName, argv[x], PATH_MAX);
 			romFileName[PATH_MAX-1] = 0;
+			if( !check_file(romFileName) ) unrecognized = 1;
 			engineState = PGS_ReloadRom;
 			break;
 		}
@@ -68,7 +126,11 @@ void parse_cmd_line(int argc, char *argv[])
 
 	if (unrecognized) {
 		printf("\n\n\nPicoDrive for Maemo v" VERSION " (c) javicq, 2010\n");
-		printf("usage: %s [options] [romfile]\n", argv[0]);
+		printf("usage: %s [options] <romfile>\n", argv[0]);
+		printf("\nAvailable options:\n");
+		printf("\t-o <config_file>\tLoad options from specified config_file. Default is picodrive.ini\n\n");
+	} else {
+		cli_config( conf_file );
 	}
 }
 
@@ -108,16 +170,6 @@ static gint dbus_callback( const gchar *interface, const gchar *method, GArray *
 	return OSSO_OK;
 }
 
-static gboolean ossoTimeoutCallback( gpointer data )
-{
-	if ( engineState == PGS_Menu ) {
-		// Assume that after N seconds we're not going to get a startup reason.
-		maemo_quit();
-	}
-
-	return FALSE; // This is a timeout, don't call us ever again.
-}
-
 static void hw_callback( osso_hw_state_t *state, gpointer data )
 {
 	if( state->shutdown_ind ) {
@@ -191,7 +243,7 @@ static void osso_config() {
 	char* rom = gconf_client_get_string( gcc, kGConfRomFile, NULL );
 	if( rom ) {
 		strcpy( romFileName, rom );
-		printf( "Loading ROM %s...", romFileName );
+		printf( "Loading ROM %s...\n", romFileName );
 	}
 
 	if( gconf_client_get_bool( gcc, kGConfSound, NULL ) )
@@ -208,8 +260,9 @@ static void osso_config() {
 	currentConfig.scaling = gconf_client_get_int( gcc, kGConfScaler, NULL );
 
 	currentConfig.screensaver = gconf_client_get_bool( gcc, kGConfSaver, NULL );
+	currentConfig.displayTS = gconf_client_get_bool( gcc, GPLAYER1_K kGConfPlayerTouchscreenShow, NULL ) || gconf_client_get_bool( gcc, GPLAYER2_K kGConfPlayerTouchscreenShow, NULL );
 
-	memset( key_map, 0, sizeof(key_map) );
+	unsigned short* key_map = currentConfig.player[0].keymap;
 	key_map[ gconf_client_get_int( gcc, GPLAYER1_KEY "a", NULL) & 0xFF ] = MDB_A;
 	key_map[ gconf_client_get_int( gcc, GPLAYER1_KEY "b", NULL) & 0xFF ] = MDB_B;
 	key_map[ gconf_client_get_int( gcc, GPLAYER1_KEY "c", NULL) & 0xFF ] = MDB_C;
@@ -236,27 +289,33 @@ static void osso_config() {
 	}
 
 	int accelSensitivities[] = { 300, 150, 75 };
+	currentConfig.player[0].inputFlags =
+		( gconf_client_get_bool( gcc, GPLAYER1_K kGConfPlayerAccelerometerEnable, NULL ) ? _P_ENABLE_ACC : 0 ) |
+		( gconf_client_get_bool( gcc, GPLAYER1_K kGConfPlayerTouchscreenEnable, NULL ) ? _P_ENABLE_TS : 0 ) |
+		( gconf_client_get_bool( gcc, GPLAYER1_K kGConfPlayerKeyboardEnable, NULL ) ? _P_ENABLE_KEY : 0 );
+	currentConfig.player[0].accelSensitivity = accelSensitivities[ gconf_client_get_int( gcc, GPLAYER1_K kGConfPlayerAccelerometerSensitivity, NULL ) ];
 
-	currentConfig.enableAccel = gconf_client_get_bool( gcc, GPLAYER1_K kGConfPlayerAccelerometerEnable, NULL );
-	currentConfig.enableTS = gconf_client_get_bool( gcc, GPLAYER1_K kGConfPlayerTouchscreenEnable, NULL );
-	currentConfig.displayTS = gconf_client_get_bool( gcc, GPLAYER1_K kGConfPlayerTouchscreenShow, NULL );
-	currentConfig.enableKey = gconf_client_get_bool( gcc, GPLAYER1_K kGConfPlayerKeyboardEnable, NULL );
-	currentConfig.accelSensitivity = accelSensitivities[ gconf_client_get_int( gcc, GPLAYER1_K kGConfPlayerAccelerometerSensitivity, NULL ) ];
-	/*gchar* zero = gconf_client_get_string( gcc, GPLAYER1_K kGConfPlayerAccelerometerZero, NULL );
-	if( zero ) {
-		sscanf( zero,
-				"%d %d %d",
-				&currentConfig.accelZeroX,
-				&currentConfig.accelZeroY,
-				&currentConfig.accelZeroZ
-				);
-	} else {
-		currentConfig.accelZeroX = currentConfig.accelZeroY = 0;
-		currentConfig.accelZeroZ = -1000;
-	}*/
-
-	guint timeout = g_timeout_add_seconds(3, ossoTimeoutCallback, 0);
-	g_warn_if_fail(timeout > 0);
+	key_map = currentConfig.player[1].keymap;
+	key_map[ gconf_client_get_int( gcc, GPLAYER2_KEY "a", NULL) & 0xFF ] = MDB_A;
+	key_map[ gconf_client_get_int( gcc, GPLAYER2_KEY "b", NULL) & 0xFF ] = MDB_B;
+	key_map[ gconf_client_get_int( gcc, GPLAYER2_KEY "c", NULL) & 0xFF ] = MDB_C;
+	key_map[ gconf_client_get_int( gcc, GPLAYER2_KEY "x", NULL) & 0xFF ] = MDB_X;
+	key_map[ gconf_client_get_int( gcc, GPLAYER2_KEY "y", NULL) & 0xFF ] = MDB_Y;
+	key_map[ gconf_client_get_int( gcc, GPLAYER2_KEY "z", NULL) & 0xFF ] = MDB_Z;
+	key_map[ gconf_client_get_int( gcc, GPLAYER2_KEY "dn", NULL) & 0xFF ] = MDB_UP;
+	key_map[ gconf_client_get_int( gcc, GPLAYER2_KEY "ds", NULL) & 0xFF ] = MDB_DOWN;
+	key_map[ gconf_client_get_int( gcc, GPLAYER2_KEY "de", NULL) & 0xFF ] = MDB_RIGHT;
+	key_map[ gconf_client_get_int( gcc, GPLAYER2_KEY "dw", NULL) & 0xFF ] = MDB_LEFT;
+	key_map[ gconf_client_get_int( gcc, GPLAYER2_KEY "start", NULL) & 0xFF ] = MDB_START;
+	key_map[ gconf_client_get_int( gcc, GPLAYER2_KEY "dne", NULL) & 0xFF ] = MDB_UP | MDB_RIGHT;
+	key_map[ gconf_client_get_int( gcc, GPLAYER2_KEY "dnw", NULL) & 0xFF ] = MDB_UP | MDB_LEFT;
+	key_map[ gconf_client_get_int( gcc, GPLAYER2_KEY "dse", NULL) & 0xFF ] = MDB_DOWN | MDB_RIGHT;
+	key_map[ gconf_client_get_int( gcc, GPLAYER2_KEY "dsw", NULL) & 0xFF ] = MDB_DOWN | MDB_LEFT;
+	currentConfig.player[1].inputFlags =
+		( gconf_client_get_bool( gcc, GPLAYER2_K kGConfPlayerAccelerometerEnable, NULL ) ? _P_ENABLE_ACC : 0 ) |
+		( gconf_client_get_bool( gcc, GPLAYER2_K kGConfPlayerTouchscreenEnable, NULL ) ? _P_ENABLE_TS : 0 ) |
+		( gconf_client_get_bool( gcc, GPLAYER2_K kGConfPlayerKeyboardEnable, NULL ) ? _P_ENABLE_KEY : 0 );
+	currentConfig.player[1].accelSensitivity = accelSensitivities[ gconf_client_get_int( gcc, GPLAYER2_K kGConfPlayerAccelerometerSensitivity, NULL ) ];
 }
 
 gboolean osso_events( gboolean block ) {
@@ -267,6 +326,7 @@ gboolean osso_events( gboolean block ) {
 }
 
 void sig_handler( int signal ) {
+	printf( "Caught signal %d, exiting...\n", signal );
 	osso_deinit();
 	exit(1);
 }
@@ -277,7 +337,7 @@ int main(int argc, char *argv[])
 	putenv("SDL_VIDEO_X11_WMCLASS=picodrive");
 	g_argv = argv;
 
-	chdir( "/opt/picodrive" );
+	//chdir( "/opt/picodrive" );
 	engineState = PGS_ReloadRom;
 	emu_setDefaultConfig();
 
@@ -289,7 +349,7 @@ int main(int argc, char *argv[])
 	}
 
 	if( !check_file(romFileName) ) {
-		fprintf( stderr, "No ROM loaded, exiting" );
+		fprintf( stderr, "No ROM loaded, exiting\n" );
 		osso_deinit();
 		exit(1);
 	}
@@ -301,6 +361,7 @@ int main(int argc, char *argv[])
 	signal( SIGABRT, sig_handler );
 	signal( SIGHUP, sig_handler );
 	signal( SIGTERM, sig_handler );
+
 	maemo_init();
 	//sharedmem_init();
 	emu_Init();
@@ -355,7 +416,7 @@ int main(int argc, char *argv[])
 				goto endloop;
 
 			default:
-				printf("engine got into unknown state (%i), exitting\n", engineState);
+				printf("engine got into unknown state (%i), exiting\n", engineState);
 				goto endloop;
 		}
 	}
